# -*- coding: utf-8 -*-

# project imports
from db_file_storage.storage import DatabaseFileStorage


def delete_file_if_needed(instance, filefield_name):
    """
        When editing and the filefield is a new file,
          delete the previous file (if any) from the database.
        Call this function immediately BEFORE saving the instance.
    """
    if instance.id:
        model_class = instance._base_manager.model

        # Check if there is a file for the instance in the database
        if model_class.objects.filter(pk=instance.pk).exclude(
            **{'%s__isnull' % filefield_name: True}
        ).exclude(
            **{'%s__exact' % filefield_name: ''}
        ).exists():
            old_file = getattr(
                model_class.objects.only(filefield_name).get(pk=instance.id),
                filefield_name
            )
        else:
            old_file = None

        # If there is a file, delete it if needed
        if old_file:
            # When editing and NOT changing the file,
            #   old_file.name == getattr(instance, filefield_name)
            #   returns True. In this case, the file must NOT be deleted.
            # If the file IS being changed, the comparison returns False.
            #   In this case, the old file MUST be deleted.
            if (old_file.name == getattr(instance, filefield_name)) is False:
                DatabaseFileStorage().delete(old_file.name)


def delete_file(instance, filefield_name):
    """
        Delete the file (if any) from the database.
        Call this function immediately AFTER deleting the instance.
    """
    file_instance = getattr(instance, filefield_name)
    if file_instance:
        DatabaseFileStorage().delete(file_instance.name)



# -*- coding: utf-8 -*-

# django imports
from django.conf.urls import patterns, url


urlpatterns = patterns('db_file_storage',
    url(
        r'^download/',
        'views.get_file',
        {'add_attachment_headers': True},
        name='db_file_storage.download_file'
    ),
    url(
        r'^get/',
        'views.get_file',
        {'add_attachment_headers': False},
        name='db_file_storage.get_file'
    )
)



# -*- coding: utf-8 -*-

# python imports
from urllib import urlencode

# django imports
from django.core.files.base import ContentFile
from django.core.files.storage import Storage
from django.core.urlresolvers import reverse
"""
    As of Django 1.7, the utilities in django.db.models.loading are
      deprecated (to be removed in 1.9) in favor of the the new
      application loading system. Check here:
      https://github.com/django/django/blob/1.7/django/db/models/loading.py
"""
from django import VERSION as DJ_VERSION
if DJ_VERSION[0] <= 1 and DJ_VERSION[1] <= 6:
    from django.db.models.loading import get_model
else:
    from django.apps import apps
    get_model = apps.get_model


def _get_model_class(model_class_path):
    app_label, model_name = model_class_path.rsplit('.', 1)
    return get_model(app_label, model_name)


def _get_encoded_bytes_from_file(_file):
    _file.seek(0)
    return _file.read().encode('base64')


def _get_file_from_encoded_bytes(encoded_bytes):
    file_buffer = str(encoded_bytes).decode('base64')
    return ContentFile(file_buffer)


def _get_unique_filename(model_class, filename_field, filename):
    final_name = filename

    if ('.' in filename.rsplit('/', 1)[-1]):
        stem, extension = final_name.rsplit('.', 1)
    else:
        stem, extension = (final_name, '')

    append_to_name = 1
    while model_class.objects.filter(**{filename_field: final_name}).exists():
        final_name = '%s_(%s)%s' % (
            stem,
            append_to_name,
            ('.%s' % extension) if extension else ''
        )
        append_to_name += 1
    return final_name


class DatabaseFileStorage(Storage):
    def _open(self, name, mode='rb'):
        (model_class_path, content_field, filename_field,
            mimetype_field, filename) = name.split('/')
        model_class = _get_model_class(model_class_path)
        model_instance = model_class.objects.only(
            content_field, mimetype_field
        ).get(**{filename_field: name})
        encoded_bytes = getattr(model_instance, content_field)
        _file = _get_file_from_encoded_bytes(encoded_bytes)
        _file.filename = filename
        _file.mimetype = getattr(model_instance, mimetype_field)
        return _file

    def _save(self, name, content):
        (model_class_path, content_field, filename_field,
            mimetype_field, filename) = name.split('/')
        model_class = _get_model_class(model_class_path)
        new_filename = _get_unique_filename(model_class, filename_field, name)
        encoded_bytes = _get_encoded_bytes_from_file(content)
        mimetype = getattr(content.file, 'content_type', 'text/plain')
        model_class.objects.create(**{
            content_field: encoded_bytes,
            filename_field: new_filename,
            mimetype_field: mimetype,
        })
        return new_filename

    def delete(self, name):
        (model_class_path, content_field, filename_field,
            mimetype_field, filename) = name.split('/')
        model_class = _get_model_class(model_class_path)
        model_class.objects.filter(**{filename_field: name}).delete()

    def exists(self, name):
        (model_class_path, content_field, filename_field,
            mimetype_field, filename) = name.split('/')
        model_class = _get_model_class(model_class_path)
        return model_class.objects.filter(
            **{filename_field: filename}
        ).exists()

    def url(self, name):
        _url = reverse('db_file_storage.download_file')
        return _url + '?' + urlencode({'name': name})



# -*- coding: utf-8 -*-

# python imports
from urllib import unquote

# django imports
from django.forms.widgets import CheckboxInput, ClearableFileInput
from django.utils.encoding import force_unicode
from django.utils.html import conditional_escape, escape
from django.utils.safestring import mark_safe


class DBClearableFileInput(ClearableFileInput):
    """
        Editing the download-link inner text.
    """
    def render(self, name, value, attrs=None):
        substitutions = {
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = u'%(input)s'
        substitutions['input'] = super(ClearableFileInput, self).render(
            name, value, attrs
        )

        if value and hasattr(value, "url"):
            template = self.template_with_initial
            substitutions['initial'] = u'<a href="%s">%s</a>' % (
                escape(value.url),
                escape(
                    force_unicode(
                        unquote(
                            value.url.split('%2F')[-1]  # <-- EDITED HERE
                        )
                    )
                )
            )
            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)
                substitutions['clear_checkbox_name'] = \
                    conditional_escape(checkbox_name)
                substitutions['clear_checkbox_id'] = \
                    conditional_escape(checkbox_id)
                substitutions['clear'] = CheckboxInput().render(
                    checkbox_name, False, attrs={'id': checkbox_id}
                )
                substitutions['clear_template'] = \
                    self.template_with_clear % substitutions

        return mark_safe(template % substitutions)



# -*- coding: utf-8 -*-

from distutils.core import setup


setup(
    name = 'django-db-file-storage',
    version = '0.2.7',
    author = 'Victor Oliveira da Silva',
    author_email = 'victor_o_silva@hotmail.com',
    packages = ['db_file_storage', 'db_file_storage.test'],
    url = 'http://pypi.python.org/pypi/django-db-file-storage/',
    license = 'Creative Commons Attribution-Noncommercial-Share Alike license',
    description = "Custom FILE_STORAGE for Django. Saves model's filefields in your database instead of your file system.",
    long_description = open('README.txt').read(),
    install_requires=[
        "Django",
    ],
)
